/***************************************************************************
 * Copyright 2013 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/
package kieker.webgui.common;

import java.io.File;
import java.lang.annotation.Annotation;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import kieker.analysis.AnalysisController;
import kieker.analysis.AnalysisController.AnalysisControllerWithMapping;
import kieker.analysis.AnalysisControllerThread;
import kieker.analysis.display.HtmlText;
import kieker.analysis.display.Image;
import kieker.analysis.display.PlainText;
import kieker.analysis.display.annotation.Display;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.analysis.plugin.AbstractPlugin;
import kieker.analysis.plugin.annotation.InputPort;
import kieker.analysis.plugin.annotation.OutputPort;
import kieker.analysis.plugin.annotation.Plugin;
import kieker.analysis.plugin.annotation.Property;
import kieker.analysis.plugin.annotation.RepositoryPort;
import kieker.analysis.plugin.filter.AbstractFilterPlugin;
import kieker.analysis.plugin.reader.AbstractReaderPlugin;
import kieker.analysis.repository.AbstractRepository;
import kieker.analysis.repository.annotation.Repository;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.common.logging.LogImplWebguiLogging;
import kieker.webgui.common.exception.ProjectLoadException;

/**
 * The {@link ClassAndMethodContainer} is a container which contains - as the name already suggests - various classes and methods. To be more precisely, it uses a
 * given class loader to load the equivalence of specific classes within this application to ensure that comparisons, assignments and the use of specific methods
 * will be done correctly. This is necessary as for every project within this application there will be a number of libraries which will be combined in one class
 * loader. This will result in multiple version of one and the same class and therefore in problems, if one doesn't use the correct class version.
 * 
 * @author Nils Christian Ehmke
 */
public final class ClassAndMethodContainer {

	/**
	 * This is the logger, which will be used to log messages and exceptions for this class.
	 */
	private static final Log LOG = LogFactory.getLog(ClassAndMethodContainer.class);
	/**
	 * The exception which is used if something went wrong during loading.
	 */
	private static final String MSG_LOAD_EXCEPTION = "An error occured while loading the classes and methods.";

	private Class<?> logImplWebguiLoggingClass;
	/**
	 * This is the class equivalence of {@link AnalysisControllerWithMapping}.
	 */
	private Class<?> analysisControllerWithMappingClass;
	/**
	 * This is the class equivalence of {@link AnalysisController}.
	 */
	private Class<?> analysisControllerClass;
	/**
	 * This is the class equivalence of {@link AnalysisControllerThread}.
	 */
	private Class<?> analysisControllerThreadClass;
	/**
	 * This is the class equivalence of {@link AbstractRepository}.
	 */
	private Class<?> abstractRepositoryClass;
	/**
	 * This is the class equivalence of {@link AbstractPlugin}.
	 */
	private Class<?> abstractPluginClass;
	/**
	 * This is the class equivalence of {@link AbstractFilterPlugin}.
	 */
	private Class<?> abstractFilterPluginClass;
	/**
	 * This is the class equivalence of {@link AbstractRepository}.
	 */
	private Class<?> abstractReaderPluginClass;
	/**
	 * This is the class equivalence of {@link Plugin}.
	 */
	private Class<? extends Annotation> pluginAnnotationClass;
	/**
	 * This is the class equivalence of {@link Repository}.
	 */
	private Class<? extends Annotation> repositoryAnnotationClass;
	/**
	 * This is the class equivalence of {@link Property}.
	 */
	private Class<? extends Annotation> propertyAnnotationClass;
	/**
	 * This is the class equivalence of {@link OutputPort}.
	 */
	private Class<? extends Annotation> outputPortAnnotationClass;
	/**
	 * This is the class equivalence of {@link InputPort}.
	 */
	private Class<? extends Annotation> inputPortAnnotationClass;
	/**
	 * This is the class equivalence of {@link RepositoryPort}.
	 */
	private Class<? extends Annotation> repositoryPortAnnotationClass;
	/**
	 * This is the class equivalence of {@link Display}.
	 */
	private Class<? extends Annotation> displayAnnotationClass;
	/**
	 * This is the class equivalence of {@link Image}.
	 */
	private Class<?> imageClass;
	/**
	 * This is the class equivalence of {@link PlainText}.
	 */
	private Class<?> plainTextClass;
	/**
	 * This is the class equivalence of {@link HtmlText}.
	 */
	private Class<?> htmlTextClass;
	/**
	 * This is the description()-method of the class equivalence of {@link Plugin}.
	 */
	private Method pluginDescriptionMethod;
	/**
	 * This is the description()-method of the class equivalence of {@link Repository}.
	 */
	private Method repositoryDescriptionMethod;
	/**
	 * This is the configuration()-method of the class equivalence of {@link Plugin}.
	 */
	private Method pluginConfigurationMethod;
	/**
	 * This is the description()-method of the class equivalence of {@link Repository}.
	 */
	private Method repositoryConfigurationMethod;
	/**
	 * This is the outputPorts()-method of the class equivalence of {@link Plugin}.
	 */
	private Method pluginOutputPortsMethod;
	/**
	 * This is the repositoryPorts()-method of the class equivalence of {@link Plugin}.
	 */
	private Method pluginRepositoryPortsMethod;
	/**
	 * This is the name()-method of the class equivalence of {@link Display}.
	 */
	private Method displayNameMethod;
	/**
	 * This is the name()-method of the class equivalence of {@link InputPort}.
	 */
	private Method inputPortNameMethod;
	/**
	 * This is the name()-method of the class equivalence of {@link OutputPort}.
	 */
	private Method outputPortNameMethod;
	/**
	 * This is the name()-method of the class equivalence of {@link RepositoryPort}.
	 */
	private Method repositoryPortNameMethod;
	/**
	 * This is the name()-method of the class equivalence of {@link Property}.
	 */
	private Method propertyNameMethod;
	/**
	 * This is the defaultValue()-method of the class equivalence of {@link Property}.
	 */
	private Method propertyDefaultValueMethod;
	/**
	 * This is the getText()-method of the class equivalence of {@link PlainText}.
	 */
	private Method plainTextgetTextMethod;
	/**
	 * This is the getController()-method of the class equivalence of {@link AnalysisControllerWithMapping}.
	 */
	private Method analysisControllerWithMappingGetController;
	/**
	 * This is the getPluginMap()-method of the class equivalence of {@link AnalysisControllerWithMapping}.
	 */
	private Method analysisControllerWithMappingGetMapping;
	/**
	 * This is the createAnalysisController(MIProject, ClassLoader)-method of the class equivalence of {@link AnalysisControllerWithMapping}.
	 */
	private Method analysisControllerCreateAnalysisController;
	/**
	 * This is the start()-method of the class equivalence of {@link AnalysisController}.
	 */
	private Method analysisControllerThreadStart;
	/**
	 * This is the terminate()-method of the class equivalence of {@link AnalysisController}.
	 */
	private Method analysisControllerThreadTerminate;
	/**
	 * This is the join(long)-method of the class equivalence of {@link AnalysisController}.
	 */
	private Method analysisControllerThreadJoin;
	/**
	 * This is the getState()-method of the class equivalence of {@link AnalysisController}.
	 */
	private Method analysisControllerGetState;
	/**
	 * This is the description()-method of the class equivalence of {@link Property}.
	 */
	private Method propertyDescriptionMethod;
	/**
	 * This is the loadFromFile(File)-method of the class equivalence of {@link AnalysisController}.
	 */
	private Method analysisControllerLoadFromFile;
	/**
	 * This is the description()-method of the class equivalence of {@link Display}.
	 */
	private Method displayDescriptionMethod;
	/**
	 * This is the dependencies()-method of the class equivalence of {@link Plugin}.
	 */
	private Method pluginDependenciesMethod;
	/**
	 * This is the dependencies()-method of the class equivalence of {@link Repository}.
	 */
	private Method repositoryDependenciesMethod;
	/**
	 * This is the programmaticOnly()-method of the class equivalence of {@link Plugin}.
	 */
	private Method pluginProgrammaticOnlyMethod;
	/**
	 * This is the programmaticOnly()-method of the class equivalence of {@link Repository}.
	 */
	private Method repositoryProgrammaticOnlyMethod;
	private Method logImplWebguiLoggingClassGetEntriesMethod;

	/**
	 * This is the constructor for {@link AnalysisControllerThread}, which gets an instance of {@link AnalysisController}.
	 */
	private Constructor<?> analysisControllerThreadConstructor;

	/**
	 * Creates a new instance of this class, using the given class loader.
	 * 
	 * @param classLoader
	 *            The class loader which will be used to load all classes and determine their methods.
	 * @throws ProjectLoadException
	 *             If one or more of the classes or methods for this container could not be found.
	 */
	public ClassAndMethodContainer(final ClassLoader classLoader) throws ProjectLoadException {
		try {

			this.loadClasses(classLoader);
			this.loadMethods();
			this.loadSpecialCases(classLoader);
			this.loadConstructors();

		} catch (final ClassNotFoundException ex) {
			// Something went wrong. We can do nothing except throwing an exception
			ClassAndMethodContainer.LOG.error(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
			throw new ProjectLoadException(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
		} catch (final NoSuchMethodException ex) {
			// Something went wrong. We can do nothing except throwing an exception
			ClassAndMethodContainer.LOG.error(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
			throw new ProjectLoadException(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
		} catch (final SecurityException ex) {
			// Something went wrong. We can do nothing except throwing an exception
			ClassAndMethodContainer.LOG.error(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
			throw new ProjectLoadException(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
		} catch (final ClassCastException ex) {
			// Something went wrong. We can do nothing except throwing an exception
			ClassAndMethodContainer.LOG.error(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
			throw new ProjectLoadException(ClassAndMethodContainer.MSG_LOAD_EXCEPTION, ex);
		}
	}

	/**
	 * Loads the constructors.
	 * 
	 * @throws NoSuchMethodException
	 *             If something went wrong.
	 * @throws SecurityException
	 *             If something went wrong.
	 * 
	 */
	private void loadConstructors() throws SecurityException, NoSuchMethodException {
		// Now the constructors
		this.analysisControllerThreadConstructor = this.analysisControllerThreadClass.getConstructor(this.analysisControllerClass);
	}

	/**
	 * Loads the special cases.
	 * 
	 * @param classLoader
	 *            The classloader.
	 * @throws ClassNotFoundException
	 *             If something went wrong.
	 * @throws NoSuchMethodException
	 *             If something went wrong.
	 * @throws SecurityException
	 *             If something went wrong.
	 * 
	 */
	private void loadSpecialCases(final ClassLoader classLoader) throws ClassNotFoundException, SecurityException, NoSuchMethodException {
		// This is a special case as we need to load some additional classes to search for the correct method
		final Class<?> miProjectClass = classLoader.loadClass(MIProject.class.getName());
		this.analysisControllerLoadFromFile = this.analysisControllerClass.getMethod("loadFromFile", File.class);

		this.analysisControllerCreateAnalysisController = this.analysisControllerClass.getMethod("createAnalysisController", miProjectClass, ClassLoader.class);

		// Another special case as the parameter is a long
		final Method[] methods = this.analysisControllerThreadClass.getMethods();
		Method joinMethod = null;
		for (final Method method : methods) {
			if ("join".equals(method.getName())) {
				joinMethod = method;
			}
		}
		this.analysisControllerThreadJoin = joinMethod;
	}

	/**
	 * Loads the methods.
	 * 
	 * @throws NoSuchMethodException
	 *             If something went wrong.
	 * @throws SecurityException
	 *             If something went wrong.
	 */
	private void loadMethods() throws SecurityException, NoSuchMethodException {
		// The following part has to be done carefully: The methods will be loaded via the name
		this.pluginDescriptionMethod = this.pluginAnnotationClass.getMethod("description", new Class<?>[0]);
		this.repositoryDescriptionMethod = this.repositoryAnnotationClass.getMethod("description", new Class<?>[0]);
		this.pluginConfigurationMethod = this.pluginAnnotationClass.getMethod("configuration", new Class<?>[0]);
		this.repositoryConfigurationMethod = this.repositoryAnnotationClass.getMethod("configuration", new Class<?>[0]);
		this.pluginOutputPortsMethod = this.pluginAnnotationClass.getMethod("outputPorts", new Class<?>[0]);
		this.pluginRepositoryPortsMethod = this.pluginAnnotationClass.getMethod("repositoryPorts", new Class<?>[0]);
		this.displayNameMethod = this.displayAnnotationClass.getMethod("name", new Class<?>[0]);
		this.inputPortNameMethod = this.inputPortAnnotationClass.getMethod("name", new Class<?>[0]);
		this.outputPortNameMethod = this.outputPortAnnotationClass.getMethod("name", new Class<?>[0]);
		this.repositoryPortNameMethod = this.repositoryPortAnnotationClass.getMethod("name", new Class<?>[0]);
		this.propertyNameMethod = this.propertyAnnotationClass.getMethod("name", new Class<?>[0]);
		this.propertyDefaultValueMethod = this.propertyAnnotationClass.getMethod("defaultValue", new Class<?>[0]);
		this.plainTextgetTextMethod = this.plainTextClass.getMethod("getText", new Class<?>[0]);
		this.analysisControllerWithMappingGetController = this.analysisControllerWithMappingClass.getMethod("getController", new Class<?>[0]);
		this.analysisControllerWithMappingGetMapping = this.analysisControllerWithMappingClass.getMethod("getPluginMap", new Class<?>[0]);
		this.analysisControllerThreadStart = this.analysisControllerThreadClass.getMethod("start", new Class<?>[0]);
		this.analysisControllerThreadTerminate = this.analysisControllerThreadClass.getMethod("terminate", new Class<?>[0]);
		this.analysisControllerGetState = this.analysisControllerClass.getMethod("getState", new Class<?>[0]);
		this.propertyDescriptionMethod = this.propertyAnnotationClass.getMethod("description", new Class<?>[0]);
		this.displayDescriptionMethod = this.displayAnnotationClass.getMethod("description", new Class<?>[0]);
		this.pluginDependenciesMethod = this.pluginAnnotationClass.getMethod("dependencies", new Class<?>[0]);
		this.repositoryDependenciesMethod = this.repositoryAnnotationClass.getMethod("dependencies", new Class<?>[0]);
		this.pluginProgrammaticOnlyMethod = this.pluginAnnotationClass.getMethod("programmaticOnly", new Class<?>[0]);
		this.repositoryProgrammaticOnlyMethod = this.repositoryAnnotationClass.getMethod("programmaticOnly", new Class<?>[0]);
		this.logImplWebguiLoggingClassGetEntriesMethod = this.logImplWebguiLoggingClass.getMethod("getEntries", String.class);

	}

	/**
	 * Loads the classes.
	 * 
	 * @param classLoader
	 *            The classloader.
	 * @throws ClassNotFoundException
	 *             if something went wrong.
	 */
	@SuppressWarnings("unchecked")
	private void loadClasses(final ClassLoader classLoader) throws ClassNotFoundException {
		// For the first: Use the classloader the load all classes we will need.
		this.logImplWebguiLoggingClass = classLoader.loadClass(LogImplWebguiLogging.class.getName());

		this.analysisControllerWithMappingClass = classLoader.loadClass(AnalysisControllerWithMapping.class.getName());

		this.analysisControllerClass = classLoader.loadClass(AnalysisController.class.getName());
		this.analysisControllerThreadClass = classLoader.loadClass(AnalysisControllerThread.class.getName());
		this.abstractFilterPluginClass = classLoader.loadClass(AbstractFilterPlugin.class.getName());
		this.abstractReaderPluginClass = classLoader.loadClass(AbstractReaderPlugin.class.getName());
		this.abstractRepositoryClass = classLoader.loadClass(AbstractRepository.class.getName());
		this.abstractPluginClass = classLoader.loadClass(AbstractPlugin.class.getName());
		this.htmlTextClass = classLoader.loadClass(HtmlText.class.getName());
		this.plainTextClass = classLoader.loadClass(PlainText.class.getName());
		this.imageClass = classLoader.loadClass(Image.class.getName());

		// Now we load the more specific annotation classes
		this.pluginAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(Plugin.class.getName());
		this.repositoryAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(Repository.class.getName());
		this.propertyAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(Property.class.getName());
		this.outputPortAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(OutputPort.class.getName());
		this.inputPortAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(InputPort.class.getName());
		this.repositoryPortAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(RepositoryPort.class.getName());
		this.displayAnnotationClass = (Class<? extends Annotation>) classLoader.loadClass(Display.class.getName());
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#abstractFilterPluginClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getAbstractFilterPluginClass() {
		return this.abstractFilterPluginClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#abstractReaderPluginClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getAbstractReaderPluginClass() {
		return this.abstractReaderPluginClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#pluginAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getPluginAnnotationClass() {
		return this.pluginAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#repositoryAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getRepositoryAnnotationClass() {
		return this.repositoryAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#propertyAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getPropertyAnnotationClass() {
		return this.propertyAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#outputPortAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getOutputPortAnnotationClass() {
		return this.outputPortAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#inputPortAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getInputPortAnnotationClass() {
		return this.inputPortAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#repositoryPortAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getRepositoryPortAnnotationClass() {
		return this.repositoryPortAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#displayAnnotationClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<? extends Annotation> getDisplayAnnotationClass() {
		return this.displayAnnotationClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#pluginDescriptionMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPluginDescriptionMethod() {
		return this.pluginDescriptionMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#repositoryDescriptionMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getRepositoryDescriptionMethod() {
		return this.repositoryDescriptionMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#pluginConfigurationMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPluginConfigurationMethod() {
		return this.pluginConfigurationMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#repositoryConfigurationMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getRepositoryConfigurationMethod() {
		return this.repositoryConfigurationMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#pluginOutputPortsMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPluginOutputPortsMethod() {
		return this.pluginOutputPortsMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#pluginRepositoryPortsMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPluginRepositoryPortsMethod() {
		return this.pluginRepositoryPortsMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#displayNameMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getDisplayNameMethod() {
		return this.displayNameMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#abstractRepositoryClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getAbstractRepositoryClass() {
		return this.abstractRepositoryClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#abstractPluginClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getAbstractPluginClass() {
		return this.abstractPluginClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#inputPortNameMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getInputPortNameMethod() {
		return this.inputPortNameMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#outputPortNameMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getOutputPortNameMethod() {
		return this.outputPortNameMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#repositoryPortNameMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getRepositoryPortNameMethod() {
		return this.repositoryPortNameMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#propertyNameMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPropertyNameMethod() {
		return this.propertyNameMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#propertyDefaultValueMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPropertyDefaultValueMethod() {
		return this.propertyDefaultValueMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#imageClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getImageClass() {
		return this.imageClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#plainTextClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getPlainTextClass() {
		return this.plainTextClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#htmlTextClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getHtmlTextClass() {
		return this.htmlTextClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#plainTextgetTextMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPlainTextgetTextMethod() {
		return this.plainTextgetTextMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#analysisControllerWithMappingGetMapping}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getAnalysisControllerWithMappingGetMapping() {
		return this.analysisControllerWithMappingGetMapping;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#analysisControllerClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getAnalysisControllerClass() {
		return this.analysisControllerClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#analysisControllerWithMappingClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getAnalysisControllerWithMappingClass() {
		return this.analysisControllerWithMappingClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#analysisControllerThreadClass}.
	 * 
	 * @return The current value for the field.
	 */
	public Class<?> getAnalysisControllerThreadClass() {
		return this.analysisControllerThreadClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#analysisControllerCreateAnalysisController}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getAnalysisControllerCreateAnalysisController() {
		return this.analysisControllerCreateAnalysisController;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#analysisControllerWithMappingGetController}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getAnalysisControllerWithMappingGetController() {
		return this.analysisControllerWithMappingGetController;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#analysisControllerThreadStart}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getAnalysisControllerThreadStart() {
		return this.analysisControllerThreadStart;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#analysisControllerThreadTerminate}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getAnalysisControllerThreadTerminate() {
		return this.analysisControllerThreadTerminate;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#analysisControllerThreadJoin}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getAnalysisControllerThreadJoin() {
		return this.analysisControllerThreadJoin;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#analysisControllerThreadJoin}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getAnalysisControllerGetState() {
		return this.analysisControllerGetState;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#propertyDescriptionMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getPropertyDescriptionMethod() {
		return this.propertyDescriptionMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#analysisControllerThreadConstructor}.
	 * 
	 * @return The current value for the field.
	 */
	public Constructor<?> getAnalysisControllerThreadConstructor() {
		return this.analysisControllerThreadConstructor;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#analysisControllerLoadFromFile}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getAnalysisControllerLoadFromFile() {
		return this.analysisControllerLoadFromFile;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#pluginDependenciesMethod}.
	 * 
	 * @return The value of the field.
	 */
	public Method getPluginDependenciesMethod() {
		return this.pluginDependenciesMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#displayDescriptionMethod}.
	 * 
	 * @return The current value for the field.
	 */
	public Method getDisplayDescriptionMethod() {
		return this.displayDescriptionMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#repositoryDependenciesMethod}.
	 * 
	 * @return The value of the field.
	 */
	public Method getRepositoryDependenciesMethod() {
		return this.repositoryDependenciesMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#pluginProgrammaticOnlyMethod}.
	 * 
	 * @return The value of the field.
	 */
	public Method getPluginProgrammaticOnlyMethod() {
		return this.pluginProgrammaticOnlyMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#repositoryProgrammaticOnlyMethod}.
	 * 
	 * @return The value of the field.
	 */
	public Method getRepositoryProgrammaticOnlyMethod() {
		return this.repositoryProgrammaticOnlyMethod;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#logImplWebguiLoggingClass}.
	 * 
	 * @return The value of the field.
	 */
	public Class<?> getLogImplWebguiLoggingClass() {
		return this.logImplWebguiLoggingClass;
	}

	/**
	 * The getter-method for the field {@link ClassAndMethodContainer#logImplWebguiLoggingClassGetEntriesMethod}.
	 * 
	 * @return The value of the field.
	 */
	public Method getLogImplWebguiLoggingClassGetEntriesMethod() {
		return this.logImplWebguiLoggingClassGetEntriesMethod;
	}

	/**
	 * This method can be used to invoke a given method with given parameters, without having to mind about the exceptions. If an exception occurs, the given default
	 * value will be returned.
	 * 
	 * @param method
	 *            The method to be invoked.
	 * @param obj
	 *            The object on which the method will be invoked.
	 * @param defaultReturn
	 *            The default return value in case of an exception.
	 * @param values
	 *            The parameters for the method.
	 * @return The result of the invoked method if everything went well, the default value otherwise.
	 */
	public static Object invokeMethod(final Method method, final Object obj, final Object defaultReturn, final Object... values) {
		try {
			return method.invoke(obj, values);
		} catch (final IllegalAccessException ex) {
			ClassAndMethodContainer.LOG.error("An error occured during a reflection method call.", ex);
		} catch (final IllegalArgumentException ex) {
			ClassAndMethodContainer.LOG.error("An error occured during a reflection method call.", ex);
		} catch (final InvocationTargetException ex) {
			ClassAndMethodContainer.LOG.error("An error occured during a reflection method call.", ex);
		}
		return defaultReturn;
	}

	/**
	 * This method can be used to invoke a given constructor with given parameters, without having to mind about the exceptions. If an exception occurs, null will be
	 * returned.
	 * 
	 * @param constructor
	 *            The constructor to be invoked.
	 * @param values
	 *            The parameters for the method.
	 * @return The result of the invoked method if everything went well, null otherwise.
	 */
	public static Object invokeConstructor(final Constructor<?> constructor, final Object... values) {
		try {
			return constructor.newInstance(values);
		} catch (final InstantiationException ex) {
			ClassAndMethodContainer.LOG.error("An error occured during a reflection constructor call.", ex);
		} catch (final IllegalAccessException ex) {
			ClassAndMethodContainer.LOG.error("An error occured during a reflection constructor call.", ex);
		} catch (final IllegalArgumentException ex) {
			ClassAndMethodContainer.LOG.error("An error occured during a reflection constructor call.", ex);
		} catch (final InvocationTargetException ex) {
			ClassAndMethodContainer.LOG.error("An error occured during a reflection constructor call.", ex);
		}
		return null;
	}

	/**
	 * This method can be used to invoke a given method with no parameters, without having to mind about the exceptions. If an exception occurs, the given default
	 * value will be returned. A call to this method is the same as {@link ClassAndMethodContainer#invokeMethod(Method, Object, Object, Object...)} with
	 * {@code new Object[0]} as parameters.
	 * 
	 * @param method
	 *            The method to be invoked.
	 * @param obj
	 *            The object on which the method will be invoked.
	 * @param defaultReturn
	 *            The default return value in case of an exception.
	 * @return The result of the invoked method if everything went well, the default value otherwise.
	 */
	public static Object invokeMethod(final Method method, final Object obj, final Object defaultReturn) {
		return ClassAndMethodContainer.invokeMethod(method, obj, defaultReturn, new Object[0]);
	}

	/**
	 * This method can be used to invoke a given <b>class</b>-method with no parameters, without having to mind about the exceptions. If an exception occurs, the
	 * given default value will be returned. A call to this method is the same as {@link ClassAndMethodContainer#invokeMethod(Method, Object, Object, Object...)}
	 * with null as object.
	 * 
	 * @param method
	 *            The method to be invoked.
	 * @param defaultReturn
	 *            The default return value in case of an exception.
	 * @param values
	 *            The parameters for the method.
	 * @return The result of the invoked method if everything went well, the default value otherwise.
	 */
	public static Object invokeClassMethod(final Method method, final Object defaultReturn, final Object... values) {
		return ClassAndMethodContainer.invokeMethod(method, null, defaultReturn, values);
	}

}
