/***************************************************************************
 * Copyright 2013 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.domain;

import java.util.Map;

import kieker.analysis.model.analysisMetaModel.MIAnalysisMetaModelFactory;
import kieker.analysis.model.analysisMetaModel.MIDisplay;
import kieker.analysis.model.analysisMetaModel.MIFilter;
import kieker.analysis.model.analysisMetaModel.MIInputPort;
import kieker.analysis.model.analysisMetaModel.MIOutputPort;
import kieker.analysis.model.analysisMetaModel.MIPlugin;
import kieker.analysis.model.analysisMetaModel.MIProperty;
import kieker.analysis.model.analysisMetaModel.MIReader;
import kieker.analysis.model.analysisMetaModel.MIRepositoryConnector;

/**
 * This is a container to wrap a single instance of {@link MIPlugin} together with different meta information.
 * 
 * @author Nils Christian Ehmke
 */
public class PluginContainer implements IComponentContainer {

	private final Map<String, String> propertyDescriptions;
	private final Map<String, String> displayDescriptions;
	private final boolean fullyInitialized;
	private final MIPlugin plugin;
	private final String description;
	private final String dependency;

	/**
	 * Creates a new instance of this class using the given parameters.
	 * 
	 * @param plugin
	 *            The plugin to be stored in this container.
	 * @param description
	 *            The description of the plugin.
	 * @param dependency
	 *            The dependency description of the plugin.
	 * @param fullyInitialized
	 *            A flag to determine whether the plugin has been initialized fully or whether there was an error during the class loading.
	 * @param propertyDescriptions
	 *            A map containing the descriptions of the properties of the plugin.
	 * @param displayDescriptions
	 *            A map containing the display descriptions of the properties of the plugin.
	 */
	public PluginContainer(final MIPlugin plugin, final String description, final String dependency, final boolean fullyInitialized,
			final Map<String, String> propertyDescriptions, final Map<String, String> displayDescriptions) {
		this.plugin = plugin;
		this.description = description;
		this.dependency = dependency;
		this.fullyInitialized = fullyInitialized;
		this.propertyDescriptions = propertyDescriptions;
		this.displayDescriptions = displayDescriptions;
	}

	/**
	 * Delivers the description of the repository.
	 * 
	 * @return The human readable description string.
	 */
	public String getDescription() {
		return this.description;
	}

	/**
	 * Delivers the dependency description of the repository.
	 * 
	 * @return The human readable dependency string.
	 */
	public String getDependency() {
		return this.dependency;
	}

	/**
	 * Tells whether the repository has been fully initialized.
	 * 
	 * @return true iff the repository has been initialized fully.
	 */
	public boolean isFullyInitialized() {
		return this.fullyInitialized;
	}

	/**
	 * Tells whether the instance is a reader or not.
	 * 
	 * @return true iff the plugin is a reader
	 */
	public boolean isReader() {
		return this.plugin instanceof MIReader;
	}

	/**
	 * Tells whether the instance is a filter or not.
	 * 
	 * @return true iff the filter is a reader
	 */
	public boolean isFilter() {
		return this.plugin instanceof MIFilter;
	}

	/**
	 * Delivers the description of the given display.
	 * 
	 * @param display
	 *            The name of the display.
	 * @return The human readable description of the given display.
	 */
	public String getDisplayDescription(final String display) {
		return this.displayDescriptions.get(display);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.common.IComponentContainer#getPropertyDescription(java.lang.String)
	 */
	@Override
	public String getPropertyDescription(final String property) {
		return this.propertyDescriptions.get(property);
	}

	/**
	 * Getter for the property {@link PluginContainer#plugin}. <b>DO NOT MODIFIY THIS OBJECT!</b> Use {@link PluginContainer#newInstance(MIAnalysisMetaModelFactory)}
	 * instead to get a copy.
	 * 
	 * @return The current value of the property.
	 */
	public MIPlugin getPlugin() {
		return this.plugin;
	}

	/**
	 * Delivers a copy of the plugin instance within this container.
	 * 
	 * @param factory
	 *            The factory to be used to create the copy.
	 * @return A deep copy of the plugin.
	 */
	public MIPlugin newInstance(final MIAnalysisMetaModelFactory factory) {
		final MIPlugin pluginCopy;

		if (this.plugin instanceof MIReader) {
			pluginCopy = factory.createReader();
		} else if (this.plugin instanceof MIFilter) {
			pluginCopy = factory.createFilter();
			// Copy the input ports of the plugin instance
			for (final MIInputPort inputPort : ((MIFilter) this.plugin).getInputPorts()) {
				final MIInputPort inputPortCopy = factory.createInputPort();
				inputPortCopy.setName(inputPort.getName());
				inputPortCopy.setParent((MIFilter) pluginCopy);
				((MIFilter) pluginCopy).getInputPorts().add(inputPortCopy);
			}
		} else {
			// This should not happen
			return null;
		}

		// Copy the output ports of the plugin instance
		for (final MIOutputPort outputPort : this.plugin.getOutputPorts()) {
			final MIOutputPort outputPortCopy = factory.createOutputPort();
			outputPortCopy.setName(outputPort.getName());
			outputPortCopy.setParent(pluginCopy);
			pluginCopy.getOutputPorts().add(outputPortCopy);
		}

		// Copy the repository "ports"
		for (final MIRepositoryConnector repositoryConnector : this.plugin.getRepositories()) {
			final MIRepositoryConnector repositoryConnectorCopy = factory.createRepositoryConnector();
			repositoryConnectorCopy.setName(repositoryConnector.getName());
			pluginCopy.getRepositories().add(repositoryConnectorCopy);
		}

		// Copy the displays
		for (final MIDisplay display : this.plugin.getDisplays()) {
			final MIDisplay displayCopy = factory.createDisplay();
			displayCopy.setName(display.getName());
			displayCopy.setParent(pluginCopy);
			pluginCopy.getDisplays().add(displayCopy);
		}

		// Copy the properties
		for (final MIProperty property : this.plugin.getProperties()) {
			final MIProperty propertyCopy = factory.createProperty();
			propertyCopy.setName(property.getName());
			propertyCopy.setValue(property.getValue());
			pluginCopy.getProperties().add(propertyCopy);
		}

		// Copy the remaining attributes
		pluginCopy.setClassname(this.plugin.getClassname());
		pluginCopy.setName(this.plugin.getName());

		return pluginCopy;
	}

}
