/***************************************************************************
 * Copyright 2013 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.persistence.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.datasource.DataSourceUtils;
import org.springframework.stereotype.Service;

import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.common.exception.DataAccessException;
import kieker.webgui.domain.User;
import kieker.webgui.domain.User.Role;
import kieker.webgui.persistence.IUserDAO;

/**
 * An implementation of the {@link IUserDAO} interface, which uses Apache Derby to store and manage the available users. A transaction manager is not necessary, as
 * all of the used commands are atomic (except for the getUsers() method, which is read only though).
 * 
 * @author Nils Christian Ehmke
 */
@Service
public class DerbyUserDAOImpl implements IUserDAO {

	private static final Log LOG = LogFactory.getLog(DerbyUserDAOImpl.class);

	@Autowired
	private DataSource dataSource;

	/**
	 * Default constructor. <b>Do not use this constructor. This bean is Spring managed.</b>
	 */
	public DerbyUserDAOImpl() {
		// No code necessary
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IserDAO#addUser(user)
	 */
	@Override
	public void addUser(final User user) throws DataAccessException {
		final Connection connection = DataSourceUtils.getConnection(this.dataSource);
		PreparedStatement userCmd = null;
		try {
			userCmd = connection.prepareStatement("INSERT INTO Users (name, password, isGuest, isUser, isAdministrator, isEnabled) VALUES (?, ?, ?, ?, ?, ?)");

			// Use all properties of the given object
			userCmd.setString(1, user.getName());
			userCmd.setString(2, user.getPassword());
			userCmd.setBoolean(3, user.getRole() == Role.ROLE_GUEST);
			userCmd.setBoolean(4, user.getRole() == Role.ROLE_USER);
			userCmd.setBoolean(5, user.getRole() == Role.ROLE_ADMIN);
			userCmd.setBoolean(6, user.isEnabled());

			// Run the command
			userCmd.execute();
		} catch (final SQLException ex) {
			// Something went wrong. Inform the calling method.
			throw new DataAccessException("Could not add user to the database.", ex);
		} finally {
			// Try to close the statement. If that doesn't work then log it, but it is not necessary to inform the calling method.
			if (userCmd != null) {
				try {
					userCmd.close();
				} catch (final SQLException ex) {
					DerbyUserDAOImpl.LOG.error("Could not close prepared statement.", ex);
				}
			}

			try {
				connection.close();
			} catch (final SQLException ex) {
				DerbyUserDAOImpl.LOG.error("Could not close connection.", ex);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IserDAO#deleteUser(user)
	 */
	@Override
	public void deleteUser(final User user) throws DataAccessException {
		final Connection connection = DataSourceUtils.getConnection(this.dataSource);
		PreparedStatement delCmd = null;
		try {
			delCmd = connection.prepareStatement("DELETE FROM Users WHERE name=?");

			delCmd.setString(1, user.getName());

			delCmd.execute();
		} catch (final SQLException ex) {
			// Something went wrong. Inform the calling method.
			throw new DataAccessException("Could not delete user from the database.", ex);
		} finally {
			// Try to close the statement. If that doesn't work then log it, but it is not necessary to inform the calling method.
			if (delCmd != null) {
				try {
					delCmd.close();
				} catch (final SQLException ex) {
					DerbyUserDAOImpl.LOG.error("Could not close prepared statement.", ex);
				}
			}

			try {
				connection.close();
			} catch (final SQLException ex) {
				DerbyUserDAOImpl.LOG.error("Could not close connection.", ex);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IserDAO#editUser(user)
	 */
	@Override
	public void editUser(final User user) throws DataAccessException {
		final Connection connection = DataSourceUtils.getConnection(this.dataSource);
		PreparedStatement updateCmd = null;
		try {
			// Choose the right update command, depending on whether the password has to be changed or not
			if (user.getPassword() == null) {
				updateCmd = connection.prepareStatement("UPDATE Users SET name=?, isGuest=?, isUser=?, isAdministrator=?, isEnabled=? WHERE name=?");
				updateCmd.setString(6, user.getName());
			} else {
				// In this case we have to set the password as well
				updateCmd = connection.prepareStatement("UPDATE Users SET name=?, isGuest=?, isUser=?, isAdministrator=?, isEnabled=?, password=? WHERE name=?");
				updateCmd.setString(6, user.getPassword());
				updateCmd.setString(7, user.getName());
			}

			// Set the other values
			updateCmd.setString(1, user.getName());
			updateCmd.setBoolean(2, user.getRole() == Role.ROLE_GUEST);
			updateCmd.setBoolean(3, user.getRole() == Role.ROLE_USER);
			updateCmd.setBoolean(4, user.getRole() == Role.ROLE_ADMIN);
			updateCmd.setBoolean(5, user.isEnabled());

			// Run the command
			updateCmd.execute();
		} catch (final SQLException ex) {
			throw new DataAccessException("Could not update user within the database.", ex);
		} finally {
			// Try to close the statement. If that doesn't work then log it, but it is not necessary to inform the calling method.
			if (updateCmd != null) {
				try {
					updateCmd.close();
				} catch (final SQLException ex) {
					DerbyUserDAOImpl.LOG.error("Could not close prepared statement.", ex);
				}
			}

			try {
				connection.close();
			} catch (final SQLException ex) {
				DerbyUserDAOImpl.LOG.error("Could not close connection.", ex);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see List<User> kieker.webgui.persistence.getUsers()
	 */
	@Override
	public List<User> getUsers() throws DataAccessException {
		final Connection connection = DataSourceUtils.getConnection(this.dataSource);
		final List<User> result = new ArrayList<User>();

		ResultSet queryResult = null;
		PreparedStatement getQuery = null;

		try {
			getQuery = connection.prepareStatement("SELECT name, isGuest, isUser, isAdministrator, isEnabled FROM Users");

			queryResult = getQuery.executeQuery();
			this.queryToUsers(queryResult, result);
		} catch (final SQLException ex) {
			DerbyUserDAOImpl.LOG.error("Could not receive user list.", ex);
		} finally {
			// Try to close the everything. If that doesn't work then log it, but it is not necessary to inform the calling method.
			try {
				if (queryResult != null) {
					queryResult.close();
				}
			} catch (final SQLException ex) {
				DerbyUserDAOImpl.LOG.error("Could not close query result.", ex);
			}
			if (getQuery != null) {
				try {
					getQuery.close();
				} catch (final SQLException ex) {
					DerbyUserDAOImpl.LOG.error("Could not close prepared statement.", ex);
				}
			}

			try {
				connection.close();
			} catch (final SQLException ex) {
				DerbyUserDAOImpl.LOG.error("Could not close connection.", ex);
			}
		}

		return result;
	}

	/**
	 * Transforms the given query result into the single users.
	 * 
	 * @param queryResult
	 *            The query result.
	 * @param result
	 *            The list which will be filled with users.
	 * @throws SQLException
	 *             If something went wrong.
	 */
	private void queryToUsers(final ResultSet queryResult, final List<User> result) throws SQLException {
		// Run through all results
		while (queryResult.next()) {
			final String username = queryResult.getString(1);
			final boolean isGuest = queryResult.getBoolean(2);
			final boolean isUser = queryResult.getBoolean(3);
			final boolean isAdministrator = queryResult.getBoolean(4);
			final boolean isEnabled = queryResult.getBoolean(5);

			// The case that the user has no role cannot happen, as the database should make sure that this is not possible
			final Role role = isAdministrator ? Role.ROLE_ADMIN : (isUser ? Role.ROLE_USER : (isGuest ? Role.ROLE_GUEST : null)); // NOPMD (Null assigning)

			result.add(new User(username, null, role, isEnabled));
		}
	}
}
