/***************************************************************************
 * Copyright 2013 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.web.beans.view;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.common.exception.DataAccessException;
import kieker.webgui.domain.User;
import kieker.webgui.domain.User.Role;
import kieker.webgui.service.IUserService;
import kieker.webgui.web.beans.application.GlobalPropertiesBean;

/**
 * The {@link CurrentUserManagementBean} contains the necessary data behind an instance of the user management page.
 * The class is a Spring managed bean with view scope to make sure that one user (even in one session) can open multiple instances of the page at a time without
 * causing any problems.
 * 
 * @author Nils Christian Ehmke
 */
@Component
@Scope("view")
public final class CurrentUserManagementBean {

	private static final Log LOG = LogFactory.getLog(CurrentUserManagementBean.class);

	@Autowired
	private IUserService userService;

	private List<User> availableUsers = new ArrayList<User>();
	private User selectedUserCopy;
	private User selectedUser;

	/**
	 * Default constructor. <b>Do not call this constructor manually. It will only be accessed by Spring.</b>
	 */
	public CurrentUserManagementBean() {
		// No code necessary
	}

	/**
	 * This method initializes the bean. <b>Do not call this method manually. It will only be accessed by Spring.</b>
	 */
	@PostConstruct
	protected synchronized void initialialize() {
		this.updateList();
	}

	/**
	 * This method adds the given user to the database and informs about success via the growl component.
	 * 
	 * @param username
	 *            The name of the new user.
	 * @param password
	 *            The password of the new user.
	 * @param role
	 *            The role of the new user.
	 * @param isEnabled
	 *            Determines whether the user is enabled already or not.
	 */
	public synchronized void addUser(final String username, final String password, final Role role, final boolean isEnabled) {
		final User user = new User(username, password, role, isEnabled);
		try {
			// Add the user to the database and - if that didn't fail - to our list.
			this.userService.addUser(user);
			this.availableUsers.add(user);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_INFO, "Added user to the database.");
		} catch (final DataAccessException ex) {
			CurrentUserManagementBean.LOG.error("Could not add the user to the database.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "Could not add the user to the database.");
		}
	}

	/**
	 * This method removes the currently selected user from the database and informs about success via the growl component.
	 */
	public synchronized void deleteUser() {
		try {
			this.userService.deleteUser(this.selectedUser);
			this.availableUsers.remove(this.selectedUser);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_INFO, "Deleted user from the database.");
		} catch (final DataAccessException ex) {
			CurrentUserManagementBean.LOG.error("Could not delete the user from the database.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "Could not delete the user from the database.");
		}
	}

	/**
	 * This method edits the currently selected user within the database and informs about success via the growl component. The password will not be changed.
	 */
	public synchronized void editUser() {
		try {
			// This is the "normal" edit - make sure that the password won't be changed
			this.selectedUserCopy.setPassword(null);
			// Edit the user and replace the entry within our list
			this.userService.editUser(this.selectedUserCopy);
			this.availableUsers.set(this.availableUsers.indexOf(this.selectedUser), this.selectedUserCopy);

			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_INFO, "Modified user within the database.");
		} catch (final DataAccessException ex) {
			CurrentUserManagementBean.LOG.error("Could not modify the user within the database.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "Could not modify the user within the database.");
		}
	}

	/**
	 * This method edits the currently selected user within the database and informs about success via the growl component. The password will be changed.
	 */
	public synchronized void editUserWithPassword() {
		try {
			// Edit the user completely and replace the entry within our list
			this.userService.editUser(this.selectedUserCopy);
			this.availableUsers.set(this.availableUsers.indexOf(this.selectedUser), this.selectedUserCopy);

			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_INFO, "Modified user within the database.");
		} catch (final DataAccessException ex) {
			CurrentUserManagementBean.LOG.error("Could not modify the user within the database.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "Could not modify the user within the database.");
		}
	}

	/**
	 * This method changes the enabled state of the given user and informs about success via the growl component.
	 * 
	 * @param user
	 *            The user whose state should be changed.
	 */
	public synchronized void changeUserEnableState(final User user) {
		try {
			this.setSelectedUser(user);
			this.selectedUserCopy.setEnabled(!this.selectedUserCopy.isEnabled());
			this.selectedUserCopy.setPassword(null);
			this.userService.editUser(this.selectedUserCopy);

			this.availableUsers.set(this.availableUsers.indexOf(this.selectedUser), this.selectedUserCopy);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_INFO, "Modified user within the database.");
		} catch (final DataAccessException ex) {
			CurrentUserManagementBean.LOG.error("Could not modify the user within the database.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "Could not modify the user within the database.");
		}
	}

	/**
	 * Updates the list containing the available users.
	 */
	public synchronized void updateList() {
		try {
			this.availableUsers = this.userService.getUsers();
		} catch (final DataAccessException ex) {
			CurrentUserManagementBean.LOG.error("An error occured while accessing the database.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "An error occured while accessing the database.");
		}
	}

	/**
	 * This method delivers all available users.
	 * 
	 * @return All currently available users.
	 */
	public synchronized List<User> getUsers() {
		return this.availableUsers;
	}

	/**
	 * Getter for the property {@link CurrentUserManagementBean#selectedUser}.
	 * 
	 * @return The current value of the property.
	 */
	public synchronized User getSelectedUser() {
		return this.selectedUser;
	}

	/**
	 * Getter for the property {@link CurrentUserManagementBean#selectedUserCopy}.
	 * 
	 * @return The current value of the property.
	 */
	public synchronized User getSelectedUserCopy() {
		return this.selectedUserCopy;
	}

	/**
	 * Setter for the property {@link CurrentUserManagementBean#selectedUser}.
	 * 
	 * @param selectedUser
	 *            The new value of the property.
	 */
	public synchronized void setSelectedUser(final User selectedUser) {
		// We remember the selected user, but we make also a copy. This is necessary, because otherwise we would have to do something like a rollback, if for example
		// an edit within the DB fails.
		this.selectedUser = selectedUser;
		this.selectedUserCopy = new User(selectedUser.getName(), null, selectedUser.getRole(), selectedUser.isEnabled());
	}
}
